@@ -5,8 +5,6 @@ import datetime
 import itertools
 import logging
 
-import six
-
 import ydb
 from .errors import DatabaseError
 
@@ -34,15 +32,13 @@ def render_date(value):
 
 def render_datetime(value):
     # TODO: is there a better solution for this?
-    return "DateTime::MakeDatetime(DateTime::ParseIso8601({}))".format(
-        render_str(value.isoformat())
-    )
+    return "DateTime::MakeDatetime(DateTime::ParseIso8601({}))".format(render_str(value.isoformat()))
 
 
 def render(value):
     if value is None:
         return "NULL"
-    if isinstance(value, six.string_types):
+    if isinstance(value, str):
         return render_str(value)
     if isinstance(value, datetime.datetime):
         return render_datetime(value)
@@ -61,9 +57,7 @@ def render_sql(sql, parameters):
     quoted_params += [""]
     sql_pieces = sql.split("?")
     assert len(sql_pieces) == len(quoted_params)
-    return "".join(
-        piece for pair in zip(sql_pieces, quoted_params) for piece in pair if piece
-    )
+    return "".join(piece for pair in zip(sql_pieces, quoted_params) for piece in pair if piece)
 
 
 def named_result_for(column_names):
