@@ -1,10 +1,18 @@
 import enum
+import typing
 
 from . import _apis
 
 from . import settings_impl as s_impl
-from ydb._grpc.common.protos import ydb_export_pb2
-from ydb._grpc.common import ydb_export_v1_pb2_grpc
+
+# Workaround for good IDE and universal for runtime
+if typing.TYPE_CHECKING:
+    from ._grpc.v4.protos import ydb_export_pb2
+    from ._grpc.v4 import ydb_export_v1_pb2_grpc
+else:
+    from ._grpc.common.protos import ydb_export_pb2
+    from ._grpc.common import ydb_export_v1_pb2_grpc
+
 from . import operation
 
 _ExportToYt = "ExportToYt"
@@ -123,7 +131,7 @@ class ExportToS3Settings(s_impl.BaseRequestSettings):
         self.secret_key = None
         self.number_of_retries = 0
         self.storage_class = None
-        self.compression = None
+        self.export_compression = None
 
     def with_scheme(self, scheme):
         self.scheme = scheme
@@ -133,8 +141,8 @@ class ExportToS3Settings(s_impl.BaseRequestSettings):
         self.storage_class = storage_class
         return self
 
-    def with_compression(self, compression):
-        self.compression = compression
+    def with_export_compression(self, compression):
+        self.export_compression = compression
         return self
 
     def with_bucket(self, bucket):
@@ -175,9 +183,7 @@ class ExportToS3Settings(s_impl.BaseRequestSettings):
 
 def _export_to_yt_request_factory(settings):
     request = ydb_export_pb2.ExportToYtRequest(
-        settings=ydb_export_pb2.ExportToYtSettings(
-            host=settings.host, token=settings.token
-        )
+        settings=ydb_export_pb2.ExportToYtSettings(host=settings.host, token=settings.token)
     )
 
     if settings.number_of_retries > 0:
@@ -187,9 +193,7 @@ def _export_to_yt_request_factory(settings):
         request.settings.port = settings.port
 
     for source_path, destination_path in settings.items:
-        request.settings.items.add(
-            source_path=source_path, destination_path=destination_path
-        )
+        request.settings.items.add(source_path=source_path, destination_path=destination_path)
 
     return request
 
@@ -217,8 +221,8 @@ def _export_to_s3_request_factory(settings):
     if settings.number_of_retries > 0:
         request.settings.number_of_retries = settings.number_of_retries
 
-    if settings.compression is not None:
-        request.settings.compression = settings.compression
+    if settings.export_compression is not None:
+        request.settings.compression = settings.export_compression
 
     for source_path, destination_prefix in settings.items:
         request.settings.items.add(
