@@ -7,7 +7,6 @@ import time
 import random
 import enum
 
-import six
 from . import (
     issues,
     convert,
@@ -21,12 +20,15 @@ from . import (
     _tx_ctx_impl,
     tracing,
 )
+from ._errors import check_retriable_error
 
 try:
     from . import interceptor
 except ImportError:
     interceptor = None
 
+_default_allow_split_transaction = False
+
 logger = logging.getLogger(__name__)
 
 ##################################################################
@@ -271,9 +273,7 @@ class PartitioningPolicy(object):
                     split_point_type.add_element(column_types[pk_name])
 
                 typed_value.type.MergeFrom(split_point_type.proto)
-                typed_value.value.MergeFrom(
-                    convert.from_native_value(split_point_type.proto, split_point.value)
-                )
+                typed_value.value.MergeFrom(convert.from_native_value(split_point_type.proto, split_point.value))
 
         return self._pb
 
@@ -445,9 +445,7 @@ class TableProfile(object):
             pb.compaction_policy.MergeFrom(self.compaction_policy.to_pb())
 
         if self.partitioning_policy is not None:
-            pb.partitioning_policy.MergeFrom(
-                self.partitioning_policy.to_pb(table_description)
-            )
+            pb.partitioning_policy.MergeFrom(self.partitioning_policy.to_pb(table_description))
 
         return pb
 
@@ -497,17 +495,11 @@ class TtlSettings(object):
         self.value_since_unix_epoch = None
 
     def with_date_type_column(self, column_name, expire_after_seconds=0):
-        self.date_type_column = DateTypeColumnModeSettings(
-            column_name, expire_after_seconds
-        )
+        self.date_type_column = DateTypeColumnModeSettings(column_name, expire_after_seconds)
         return self
 
-    def with_value_since_unix_epoch(
-        self, column_name, column_unit, expire_after_seconds=0
-    ):
-        self.value_since_unix_epoch = ValueSinceUnixEpochModeSettings(
-            column_name, column_unit, expire_after_seconds
-        )
+    def with_value_since_unix_epoch(self, column_name, column_unit, expire_after_seconds=0):
+        self.value_since_unix_epoch = ValueSinceUnixEpochModeSettings(column_name, column_unit, expire_after_seconds)
         return self
 
     def to_pb(self):
@@ -767,8 +759,7 @@ class TableDescription(object):
         return self
 
 
-@six.add_metaclass(abc.ABCMeta)
-class AbstractTransactionModeBuilder(object):
+class AbstractTransactionModeBuilder(abc.ABC):
     @property
     @abc.abstractmethod
     def name(self):
@@ -780,6 +771,22 @@ class AbstractTransactionModeBuilder(object):
         pass
 
 
+class SnapshotReadOnly(AbstractTransactionModeBuilder):
+    __slots__ = ("_pb", "_name")
+
+    def __init__(self):
+        self._pb = _apis.ydb_table.SnapshotModeSettings()
+        self._name = "snapshot_read_only"
+
+    @property
+    def settings(self):
+        return self._pb
+
+    @property
+    def name(self):
+        return self._name
+
+
 class SerializableReadWrite(AbstractTransactionModeBuilder):
     __slots__ = ("_pb", "_name")
 
@@ -843,9 +850,7 @@ class BackoffSettings(object):
         slots_count = 1 << min(retry_number, self.ceiling)
         max_duration_ms = slots_count * self.slot_duration * 1000.0
         # duration_ms = random.random() * max_duration_ms * uncertain_ratio) + max_duration_ms * (1 - uncertain_ratio)
-        duration_ms = max_duration_ms * (
-            random.random() * self.uncertain_ratio + 1.0 - self.uncertain_ratio
-        )
+        duration_ms = max_duration_ms * (random.random() * self.uncertain_ratio + 1.0 - self.uncertain_ratio)
         return duration_ms / 1000.0
 
 
@@ -864,14 +869,8 @@ class RetrySettings(object):
     ):
         self.max_retries = max_retries
         self.max_session_acquire_timeout = max_session_acquire_timeout
-        self.on_ydb_error_callback = (
-            (lambda e: None) if on_ydb_error_callback is None else on_ydb_error_callback
-        )
-        self.fast_backoff = (
-            BackoffSettings(10, 0.005)
-            if fast_backoff_settings is None
-            else fast_backoff_settings
-        )
+        self.on_ydb_error_callback = (lambda e: None) if on_ydb_error_callback is None else on_ydb_error_callback
+        self.fast_backoff = BackoffSettings(10, 0.005) if fast_backoff_settings is None else fast_backoff_settings
         self.slow_backoff = (
             BackoffSettings(backoff_ceiling, backoff_slot_duration)
             if slow_backoff_settings is None
@@ -883,9 +882,7 @@ class RetrySettings(object):
         self.unknown_error_handler = lambda e: None
         self.get_session_client_timeout = get_session_client_timeout
         if max_session_acquire_timeout is not None:
-            self.get_session_client_timeout = min(
-                self.max_session_acquire_timeout, self.get_session_client_timeout
-            )
+            self.get_session_client_timeout = min(self.max_session_acquire_timeout, self.get_session_client_timeout)
 
     def with_fast_backoff(self, backoff_settings):
         self.fast_backoff = backoff_settings
@@ -900,12 +897,24 @@ class YdbRetryOperationSleepOpt(object):
     def __init__(self, timeout):
         self.timeout = timeout
 
+    def __eq__(self, other):
+        return type(self) == type(other) and self.timeout == other.timeout
+
+    def __repr__(self):
+        return "YdbRetryOperationSleepOpt(%s)" % self.timeout
+
 
 class YdbRetryOperationFinalResult(object):
     def __init__(self, result):
         self.result = result
         self.exc = None
 
+    def __eq__(self, other):
+        return type(self) == type(other) and self.result == other.result and self.exc == other.exc
+
+    def __repr__(self):
+        return "YdbRetryOperationFinalResult(%s, exc=%s)" % (self.result, self.exc)
+
     def set_exception(self, exc):
         self.exc = exc
 
@@ -914,7 +923,7 @@ def retry_operation_impl(callee, retry_settings=None, *args, **kwargs):
     retry_settings = RetrySettings() if retry_settings is None else retry_settings
     status = None
 
-    for attempt in six.moves.range(retry_settings.max_retries + 1):
+    for attempt in range(retry_settings.max_retries + 1):
         try:
             result = YdbRetryOperationFinalResult(callee(*args, **kwargs))
             yield result
@@ -922,56 +931,28 @@ def retry_operation_impl(callee, retry_settings=None, *args, **kwargs):
             if result.exc is not None:
                 raise result.exc
 
-        except (
-            issues.Aborted,
-            issues.BadSession,
-            issues.NotFound,
-            issues.InternalError,
-        ) as e:
+        except issues.Error as e:
             status = e
             retry_settings.on_ydb_error_callback(e)
 
-            if isinstance(e, issues.NotFound) and not retry_settings.retry_not_found:
-                raise e
-
-            if (
-                isinstance(e, issues.InternalError)
-                and not retry_settings.retry_internal_error
-            ):
-                raise e
-
-        except (
-            issues.Overloaded,
-            issues.SessionPoolEmpty,
-            issues.ConnectionError,
-        ) as e:
-            status = e
-            retry_settings.on_ydb_error_callback(e)
-            yield YdbRetryOperationSleepOpt(
-                retry_settings.slow_backoff.calc_timeout(attempt)
-            )
-
-        except issues.Unavailable as e:
-            status = e
-            retry_settings.on_ydb_error_callback(e)
-            yield YdbRetryOperationSleepOpt(
-                retry_settings.fast_backoff.calc_timeout(attempt)
-            )
-
-        except issues.Undetermined as e:
-            status = e
-            retry_settings.on_ydb_error_callback(e)
-            if not retry_settings.idempotent:
-                # operation is not idempotent, so we cannot retry.
+            retriable_info = check_retriable_error(e, retry_settings, attempt)
+            if not retriable_info.is_retriable:
                 raise
 
-            yield YdbRetryOperationSleepOpt(
-                retry_settings.fast_backoff.calc_timeout(attempt)
-            )
+            skip_yield_error_types = [
+                issues.Aborted,
+                issues.BadSession,
+                issues.NotFound,
+                issues.InternalError,
+            ]
 
-        except issues.Error as e:
-            retry_settings.on_ydb_error_callback(e)
-            raise
+            yield_sleep = True
+            for t in skip_yield_error_types:
+                if isinstance(e, t):
+                    yield_sleep = False
+
+            if yield_sleep:
+                yield YdbRetryOperationSleepOpt(retriable_info.sleep_timeout_seconds)
 
         except Exception as e:
             # you should provide your own handler you want
@@ -999,6 +980,7 @@ class TableClientSettings(object):
         self._native_json_in_result_sets = False
         self._native_interval_in_result_sets = False
         self._native_timestamp_in_result_sets = False
+        self._allow_truncated_result = convert._default_allow_truncated_result
 
     def with_native_timestamp_in_result_sets(self, enabled):
         # type:(bool) -> ydb.TableClientSettings
@@ -1035,14 +1017,17 @@ class TableClientSettings(object):
         self._make_result_sets_lazy = enabled
         return self
 
+    def with_allow_truncated_result(self, enabled):
+        # type:(bool) -> ydb.TableClientSettings
+        self._allow_truncated_result = enabled
+        return self
+
 
 class ScanQueryResult(object):
     def __init__(self, result, table_client_settings):
         self._result = result
         self.query_stats = result.query_stats
-        self.result_set = convert.ResultSet.from_message(
-            self._result.result_set, table_client_settings
-        )
+        self.result_set = convert.ResultSet.from_message(self._result.result_set, table_client_settings)
 
 
 @enum.unique
@@ -1090,8 +1075,7 @@ def _scan_query_request_factory(query, parameters=None, settings=None):
     )
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ISession:
+class ISession(abc.ABC):
     @abstractmethod
     def __init__(self, driver, table_client_settings):
         pass
@@ -1171,7 +1155,7 @@ class ISession:
         pass
 
     @abstractmethod
-    def transaction(self, tx_mode=None):
+    def transaction(self, tx_mode=None, allow_split_transactions=None):
         pass
 
     @abstractmethod
@@ -1253,8 +1237,7 @@ class ISession:
         pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ITableClient:
+class ITableClient(abc.ABC):
     def __init__(self, driver, table_client_settings=None):
         pass
 
@@ -1283,11 +1266,7 @@ class BaseTableClient(ITableClient):
     def __init__(self, driver, table_client_settings=None):
         # type:(ydb.Driver, ydb.TableClientSettings) -> None
         self._driver = driver
-        self._table_client_settings = (
-            TableClientSettings()
-            if table_client_settings is None
-            else table_client_settings
-        )
+        self._table_client_settings = TableClientSettings() if table_client_settings is None else table_client_settings
 
     def session(self):
         # type: () -> ydb.Session
@@ -1356,9 +1335,7 @@ class TableClient(BaseTableClient):
 
 
 def _make_index_description(index):
-    result = TableIndex(index.name).with_index_columns(
-        *tuple(col for col in index.index_columns)
-    )
+    result = TableIndex(index.name).with_index_columns(*tuple(col for col in index.index_columns))
     result.status = IndexStatus(index.status)
     return result
 
@@ -1389,20 +1366,10 @@ class TableSchemeEntry(scheme.SchemeEntry):
     ):
 
         super(TableSchemeEntry, self).__init__(
-            name,
-            owner,
-            type,
-            effective_permissions,
-            permissions,
-            size_bytes,
-            *args,
-            **kwargs
+            name, owner, type, effective_permissions, permissions, size_bytes, *args, **kwargs
         )
         self.primary_key = [pk for pk in primary_key]
-        self.columns = [
-            Column(column.name, convert.type_to_native(column.type), column.family)
-            for column in columns
-        ]
+        self.columns = [Column(column.name, convert.type_to_native(column.type), column.family) for column in columns]
         self.indexes = [_make_index_description(index) for index in indexes]
         self.shard_key_ranges = []
         self.column_families = []
@@ -1417,9 +1384,7 @@ class TableSchemeEntry(scheme.SchemeEntry):
             )
 
             if column_family.HasField("data"):
-                self.column_families[-1].with_data(
-                    StoragePool(column_family.data.media)
-                )
+                self.column_families[-1].with_data(StoragePool(column_family.data.media))
 
         for shard_key_bound in shard_key_bounds:
             # for next key range
@@ -1427,9 +1392,7 @@ class TableSchemeEntry(scheme.SchemeEntry):
             current_bound = convert.to_native_value(shard_key_bound)
             self.shard_key_ranges.append(
                 KeyRange(
-                    None
-                    if left_key_bound is None
-                    else KeyBound.inclusive(left_key_bound, key_bound_type),
+                    None if left_key_bound is None else KeyBound.inclusive(left_key_bound, key_bound_type),
                     KeyBound.exclusive(current_bound, key_bound_type),
                 )
             )
@@ -1462,23 +1425,15 @@ class TableSchemeEntry(scheme.SchemeEntry):
         self.storage_settings = None
         if storage_settings is not None:
             self.storage_settings = StorageSettings()
-            self.storage_settings.store_external_blobs = FeatureFlag(
-                self.storage_settings.store_external_blobs
-            )
+            self.storage_settings.store_external_blobs = FeatureFlag(self.storage_settings.store_external_blobs)
             if storage_settings.HasField("tablet_commit_log0"):
-                self.storage_settings.with_tablet_commit_log0(
-                    StoragePool(storage_settings.tablet_commit_log0.media)
-                )
+                self.storage_settings.with_tablet_commit_log0(StoragePool(storage_settings.tablet_commit_log0.media))
 
             if storage_settings.HasField("tablet_commit_log1"):
-                self.storage_settings.with_tablet_commit_log1(
-                    StoragePool(storage_settings.tablet_commit_log1.media)
-                )
+                self.storage_settings.with_tablet_commit_log1(StoragePool(storage_settings.tablet_commit_log1.media))
 
             if storage_settings.HasField("external"):
-                self.storage_settings.with_external(
-                    StoragePool(storage_settings.external.media)
-                )
+                self.storage_settings.with_external(StoragePool(storage_settings.external.media))
 
         self.partitioning_settings = None
         if partitioning_settings is not None:
@@ -1514,14 +1469,10 @@ class TableSchemeEntry(scheme.SchemeEntry):
         if table_stats is not None:
             self.table_stats = TableStats()
             if table_stats.partitions != 0:
-                self.table_stats = self.table_stats.with_partitions(
-                    table_stats.partitions
-                )
+                self.table_stats = self.table_stats.with_partitions(table_stats.partitions)
 
             if table_stats.store_size != 0:
-                self.table_stats = self.table_stats.with_store_size(
-                    table_stats.store_size
-                )
+                self.table_stats = self.table_stats.with_store_size(table_stats.store_size)
 
         self.attributes = attributes
 
@@ -1620,9 +1571,7 @@ class BaseSession(ISession):
             _apis.TableService.StreamReadTable,
             settings=settings,
         )
-        return _utilities.SyncResponseIterator(
-            stream_it, _session_impl.wrap_read_table_response
-        )
+        return _utilities.SyncResponseIterator(stream_it, _session_impl.wrap_read_table_response)
 
     def keep_alive(self, settings=None):
         return self._driver(
@@ -1641,9 +1590,7 @@ class BaseSession(ISession):
         create_settings = settings_impl.BaseRequestSettings()
         if settings is not None:
             create_settings = settings.make_copy()
-        create_settings = create_settings.with_header(
-            "x-ydb-client-capabilities", "session-balancer"
-        )
+        create_settings = create_settings.with_header("x-ydb-client-capabilities", "session-balancer")
         return self._driver(
             _apis.ydb_table.CreateSessionRequest(),
             _apis.TableService.Stub,
@@ -1676,8 +1623,14 @@ class BaseSession(ISession):
             self._state.endpoint,
         )
 
-    def transaction(self, tx_mode=None):
-        return TxContext(self._driver, self._state, self, tx_mode)
+    def transaction(self, tx_mode=None, allow_split_transactions=None):
+        return TxContext(
+            self._driver,
+            self._state,
+            self,
+            tx_mode,
+            allow_split_transactions=allow_split_transactions,
+        )
 
     def has_prepared(self, query):
         return query in self._state
@@ -1726,9 +1679,7 @@ class BaseSession(ISession):
         :return: A description of created scheme entry or error otherwise.
         """
         return self._driver(
-            _session_impl.create_table_request_factory(
-                self._state, path, table_description
-            ),
+            _session_impl.create_table_request_factory(self._state, path, table_description),
             _apis.TableService.Stub,
             _apis.TableService.CreateTable,
             _session_impl.wrap_operation,
@@ -1818,9 +1769,7 @@ class BaseSession(ISession):
 
     def copy_tables(self, source_destination_pairs, settings=None):
         return self._driver(
-            _session_impl.copy_tables_request_factory(
-                self._state, source_destination_pairs
-            ),
+            _session_impl.copy_tables_request_factory(self._state, source_destination_pairs),
             _apis.TableService.Stub,
             _apis.TableService.CopyTables,
             _session_impl.wrap_operation,
@@ -1882,9 +1831,7 @@ class Session(BaseSession):
             _apis.TableService.StreamReadTable,
             settings=settings,
         )
-        return _utilities.AsyncResponseIterator(
-            stream_it, _session_impl.wrap_read_table_response
-        )
+        return _utilities.AsyncResponseIterator(stream_it, _session_impl.wrap_read_table_response)
 
     @_utilities.wrap_async_call_exceptions
     def async_keep_alive(self, settings=None):
@@ -1905,9 +1852,7 @@ class Session(BaseSession):
         create_settings = settings_impl.BaseRequestSettings()
         if settings is not None:
             create_settings = settings.make_copy()
-        create_settings = create_settings.with_header(
-            "x-ydb-client-capabilities", "session-balancer"
-        )
+        create_settings = create_settings.with_header("x-ydb-client-capabilities", "session-balancer")
         return self._driver.future(
             _apis.ydb_table.CreateSessionRequest(),
             _apis.TableService.Stub,
@@ -1963,9 +1908,7 @@ class Session(BaseSession):
     @_utilities.wrap_async_call_exceptions
     def async_create_table(self, path, table_description, settings=None):
         return self._driver.future(
-            _session_impl.create_table_request_factory(
-                self._state, path, table_description
-            ),
+            _session_impl.create_table_request_factory(self._state, path, table_description),
             _apis.TableService.Stub,
             _apis.TableService.CreateTable,
             _session_impl.wrap_operation,
@@ -2034,16 +1977,12 @@ class Session(BaseSession):
         )
 
     def async_copy_table(self, source_path, destination_path, settings=None):
-        return self.async_copy_tables(
-            [(source_path, destination_path)], settings=settings
-        )
+        return self.async_copy_tables([(source_path, destination_path)], settings=settings)
 
     @_utilities.wrap_async_call_exceptions
     def async_copy_tables(self, source_destination_pairs, settings=None):
         return self._driver.future(
-            _session_impl.copy_tables_request_factory(
-                self._state, source_destination_pairs
-            ),
+            _session_impl.copy_tables_request_factory(self._state, source_destination_pairs),
             _apis.TableService.Stub,
             _apis.TableService.CopyTables,
             _session_impl.wrap_operation,
@@ -2077,8 +2016,7 @@ class Session(BaseSession):
         )
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ITxContext:
+class ITxContext(abc.ABC):
     @abstractmethod
     def __init__(self, driver, session_state, session, tx_mode=None):
         """
@@ -2189,9 +2127,19 @@ class ITxContext:
 
 
 class BaseTxContext(ITxContext):
-    __slots__ = ("_tx_state", "_session_state", "_driver", "session")
+    __slots__ = (
+        "_tx_state",
+        "_session_state",
+        "_driver",
+        "session",
+        "_finished",
+        "_allow_split_transactions",
+    )
 
-    def __init__(self, driver, session_state, session, tx_mode=None):
+    _COMMIT = "commit"
+    _ROLLBACK = "rollback"
+
+    def __init__(self, driver, session_state, session, tx_mode=None, *, allow_split_transactions=None):
         """
         An object that provides a simple transaction context manager that allows statements execution
         in a transaction. You don't have to open transaction explicitly, because context manager encapsulates
@@ -2214,6 +2162,8 @@ class BaseTxContext(ITxContext):
         self._tx_state = _tx_ctx_impl.TxState(tx_mode)
         self._session_state = session_state
         self.session = session
+        self._finished = ""
+        self._allow_split_transactions = allow_split_transactions
 
     def __enter__(self):
         """
@@ -2271,6 +2221,11 @@ class BaseTxContext(ITxContext):
 
         :return: A result sets or exception in case of execution errors
         """
+
+        self._check_split()
+        if commit_tx:
+            self._set_finish(self._COMMIT)
+
         return self._driver(
             _tx_ctx_impl.execute_request_factory(
                 self._session_state,
@@ -2297,8 +2252,12 @@ class BaseTxContext(ITxContext):
 
         :return: A committed transaction or exception if commit is failed
         """
+
+        self._set_finish(self._COMMIT)
+
         if self._tx_state.tx_id is None and not self._tx_state.dead:
             return self
+
         return self._driver(
             _tx_ctx_impl.commit_request_factory(self._session_state, self._tx_state),
             _apis.TableService.Stub,
@@ -2318,8 +2277,12 @@ class BaseTxContext(ITxContext):
 
         :return: A rolled back transaction or exception if rollback is failed
         """
+
+        self._set_finish(self._ROLLBACK)
+
         if self._tx_state.tx_id is None and not self._tx_state.dead:
             return self
+
         return self._driver(
             _tx_ctx_impl.rollback_request_factory(self._session_state, self._tx_state),
             _apis.TableService.Stub,
@@ -2340,6 +2303,9 @@ class BaseTxContext(ITxContext):
         """
         if self._tx_state.tx_id is not None:
             return self
+
+        self._check_split()
+
         return self._driver(
             _tx_ctx_impl.begin_request_factory(self._session_state, self._tx_state),
             _apis.TableService.Stub,
@@ -2350,6 +2316,27 @@ class BaseTxContext(ITxContext):
             self._session_state.endpoint,
         )
 
+    def _set_finish(self, val):
+        self._check_split(val)
+        self._finished = val
+
+    def _check_split(self, allow=""):
+        """
+        Deny all operaions with transaction after commit/rollback.
+        Exception: double commit and double rollbacks, because it is safe
+        """
+        allow_split_transaction = (
+            self._allow_split_transactions
+            if self._allow_split_transactions is not None
+            else _default_allow_split_transaction
+        )
+
+        if allow_split_transaction:
+            return
+
+        if self._finished != "" and self._finished != allow:
+            raise RuntimeError("Any operation with finished transaction is denied")
+
 
 class TxContext(BaseTxContext):
     @_utilities.wrap_async_call_exceptions
@@ -2365,6 +2352,9 @@ class TxContext(BaseTxContext):
 
         :return: A future of query execution
         """
+
+        self._check_split()
+
         return self._driver.future(
             _tx_ctx_impl.execute_request_factory(
                 self._session_state,
@@ -2396,8 +2386,11 @@ class TxContext(BaseTxContext):
 
         :return: A future of commit call
         """
+        self._set_finish(self._COMMIT)
+
         if self._tx_state.tx_id is None and not self._tx_state.dead:
             return _utilities.wrap_result_in_future(self)
+
         return self._driver.future(
             _tx_ctx_impl.commit_request_factory(self._session_state, self._tx_state),
             _apis.TableService.Stub,
@@ -2418,8 +2411,11 @@ class TxContext(BaseTxContext):
 
         :return: A future of rollback call
         """
+        self._set_finish(self._ROLLBACK)
+
         if self._tx_state.tx_id is None and not self._tx_state.dead:
             return _utilities.wrap_result_in_future(self)
+
         return self._driver.future(
             _tx_ctx_impl.rollback_request_factory(self._session_state, self._tx_state),
             _apis.TableService.Stub,
@@ -2441,6 +2437,9 @@ class TxContext(BaseTxContext):
         """
         if self._tx_state.tx_id is not None:
             return _utilities.wrap_result_in_future(self)
+
+        self._check_split()
+
         return self._driver.future(
             _tx_ctx_impl.begin_request_factory(self._session_state, self._tx_state),
             _apis.TableService.Stub,
@@ -2487,9 +2486,7 @@ class SessionPool(object):
         retry_settings = RetrySettings() if retry_settings is None else retry_settings
 
         def wrapped_callee():
-            with self.checkout(
-                timeout=retry_settings.get_session_client_timeout
-            ) as session:
+            with self.checkout(timeout=retry_settings.get_session_client_timeout) as session:
                 return callee(session, *args, **kwargs)
 
         return retry_operation_sync(wrapped_callee, retry_settings)
